#ifndef OEM_MSG_H
#define OEM_MSG_H

/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2012 KYOCERA Corporation
 */

/*===========================================================================

                     INCLUDE FILES FOR MODULE

===========================================================================*/

/*===========================================================================
            LOCAL DEFINITIONS AND DECLARATIONS FOR MODULE
===========================================================================*/

/*---------------------------------------------------------------------------
  These are the masks that are used to identify a message as belonging to
  group "i" of a particular Subsystem. This allows the user to selectively
  turn groups of messages within a Subsystem to ON or OFF. The legacy
  messages will continue to be supported thru the legacy masks.
---------------------------------------------------------------------------*/
#define MSG_MASK_0   (0x00000001)
#define MSG_MASK_1   (0x00000002)
#define MSG_MASK_2   (0x00000004)
#define MSG_MASK_3   (0x00000008)
#define MSG_MASK_4   (0x00000010)

/*---------------------------------------------------------------------------
  These masks are to be used for support of all legacy messages in the sw.
  The user does not need to remember the names as they will be embedded in 
  the appropriate macros.
---------------------------------------------------------------------------*/
#define MSG_LEGACY_LOW      MSG_MASK_0
#define MSG_LEGACY_MED      MSG_MASK_1
#define MSG_LEGACY_HIGH     MSG_MASK_2
#define MSG_LEGACY_ERROR    MSG_MASK_3
#define MSG_LEGACY_FATAL    MSG_MASK_4

/*---------------------------------------------------------------------------
  Legacy Message Priorities 
---------------------------------------------------------------------------*/
#define MSG_LVL_FATAL   (MSG_LEGACY_FATAL)
#define MSG_LVL_ERROR   (MSG_LEGACY_ERROR | MSG_LVL_FATAL)
#define MSG_LVL_HIGH    (MSG_LEGACY_HIGH | MSG_LVL_ERROR)
#define MSG_LVL_MED     (MSG_LEGACY_MED | MSG_LVL_HIGH)
#define MSG_LVL_LOW     (MSG_LEGACY_LOW | MSG_LVL_MED)

#define MSG_LVL_NONE    0

#define MSG_SSID_ANDROID_KERNEL     (0xC004) /* Android Kernel */

/* If MSG_FILE is defined, use that as the filename, and allocate a single
   character string to contain it.  Note that this string is shared with the
   Error Services, to conserve ROM and RAM.

   With ADS1.1 and later, multiple uses of __FILE__ or __MODULE__ within the
   same file do not cause multiple literal strings to be stored in ROM. So in
   builds that use the more recent versions of ADS, it is not necessary to
   define the static variable msg_file. Note that __MODULE__ gets replaced
   with the filename portion of the full pathname of the file. */
/* Note, if this symbol is activated, the file is being compiled with legacy
   message service defines.  Older compilers may need the static const
   variable to avoid multiple copies of the filename string in ROM. */

//#define msg_file __MODULE__
#define msg_file __FILE__

/*---------------------------------------------------------------------------
  The extended message packet is defined to be processed as efficiently as
  possible in the caller's context.  Therefore, the packet is divided into
  logical blocks that are aligned w/out declaring the structure as PACKED.
  
  A header, static constant block, and argument list are defined to minimize
  the work done by the caller's task.
   
   1. Header
   2. Constant variable length data (format string and filename string, etc).
   3. Arguments ("Variable" variable-length data)
   
  The data is delivered in this order in the packet to simplify the runtime 
  processing of each message.  All constant data is handled in DIAG task 
  context, leaving the caller's task to only process variable data at runtime.
   
  The phone will never process the constant data directly, except to copy 
  format and filename strings. 
---------------------------------------------------------------------------*/
typedef struct
{
  unsigned short line;			/* Line number in source file */
  unsigned short ss_id;			/* Subsystem ID               */
  unsigned long ss_mask;		/* Subsystem Mask             */
}
msg_desc_type;
/*---------------------------------------------------------------------------
  This is the message HEADER type. It contains the beginning fields of the 
  packet and is fixed length.  These fields are filled by the calling task.
---------------------------------------------------------------------------*/
typedef struct
{
  unsigned char cmd_code;		/* Command code */
  unsigned char ts_type;		/* Time stamp type */
  unsigned char num_args;		/* Number of arguments in message */
  unsigned char drop_cnt;		/* # dropped since last successful message */
  unsigned long ts_lo; /* Time stamp */
  unsigned long ts_hi;
}
msg_hdr_type;

/*---------------------------------------------------------------------------
  All constant information stored for a message.
   
  The values for the fields of this structure are known at compile time. 
  So this is to be defined as a "static " in the MACRO, so it ends up
  being defined and initialized at compile time for each and every message 
  in the software. This minimizes the amount of work to do during run time.
  
  So this structure is to be used in the "caller's" context. "Caller" is the
  client of the Message Services.
---------------------------------------------------------------------------*/
typedef struct
{
  msg_desc_type desc;       /* ss_mask, line, ss_id */
  const char *fmt;      /* Printf style format string */
  const char *fname;        /* Pointer to source file name */

#if defined (FEATURE_ERR_EXTENDED_STORE) && defined (FEATURE_SAVE_DEBUG_TRACE)
  boolean do_save;              /* If TRUE, save msg to RAM buffer */
#endif
}
msg_const_type;

/*---------------------------------------------------------------------------
  This is the structure that is stored by the caller's task in msg_send ().
  The DIAG task will expand the constant data into the final packet before
  sending to the external device.
---------------------------------------------------------------------------*/
typedef struct
{
  msg_hdr_type hdr;
  const msg_const_type *const_data_ptr; /* desc, file_name, fmt */
  unsigned long args[1];
}
msg_ext_store_type;



/* Enabling real diag F3 messages, Diag 1.5 */

/*---------------------------------------------------------------------------
  The purpose of this macro is to define the constant part of the message
  that can be initialized at compile time and stored in ROM. This will 
  define and initialize a msg_const_type for each call of a message macro.
  The "static" limits the scope to the file the macro is called from while
  using the macro in a do{}while() guarantees the uniqueness of the name.
---------------------------------------------------------------------------*/
#if defined (FEATURE_ERR_EXTENDED_STORE) && defined (FEATURE_SAVE_DEBUG_TRACE)
  #define XX_MSG_CONST(xx_ss_id, xx_ss_mask, xx_fmt) \
              XX_MSG_CONST_SAVE(xx_ss_id, xx_ss_mask, xx_fmt, TRUE)

  #define XX_MSG_CONST_SAVE(xx_ss_id, xx_ss_mask, xx_fmt, do_save) \
    static const msg_const_type xx_msg_const = { \
      {__LINE__, (xx_ss_id), (xx_ss_mask)}, (xx_fmt), msg_file, do_save}

#else

  #define XX_MSG_CONST(xx_ss_id, xx_ss_mask, xx_fmt) \
    static const msg_const_type xx_msg_const = { \
      {__LINE__, (xx_ss_id), (xx_ss_mask)}, (xx_fmt), msg_file}

#endif /* (FEATURE_ERR_EXTENDED_STORE) && defined (FEATURE_SAVE_DEBUG_TRACE) */

/*---------------------------------------------------------------------------
  This is the macro for messages with 3 parameters.
---------------------------------------------------------------------------*/
#define MSG_3(xx_ss_id, xx_ss_mask, xx_fmt, xx_arg1, xx_arg2, xx_arg3) \
  do { \
    /*lint +e506 +e774*/ \
      XX_MSG_CONST (xx_ss_id, xx_ss_mask, xx_fmt); \
      /*lint -e571 */ \
      msg_send_3 (&xx_msg_const, (unsigned long) (xx_arg1), (unsigned long) (xx_arg2), \
                                 (unsigned long) (xx_arg3)); \
  /*lint -e717 */ \
  } while (0) \
                       /* lint +e717 */

#define MSG_3_SAVE(xx_ss_id, xx_ss_mask, xx_fmt, xx_arg1, xx_arg2, xx_arg3, do_sav) \
  do { \
    /*lint +e506 +e774*/ \
      XX_MSG_CONST_SAVE (xx_ss_id, xx_ss_mask, xx_fmt, do_sav); \
      /*lint -e571 */ \
      msg_send_3 (&xx_msg_const, (unsigned long) (xx_arg1), (unsigned long) (xx_arg2), \
                                 (unsigned long) (xx_arg3)); \
  /*lint -e717 */ \
  } while (0) \
                       /* lint +e717 */

/*---------------------------------------------------------------------------
  The following MACROs are for LEGACY diagnostic messages support.  
---------------------------------------------------------------------------*/

/*===========================================================================

MACRO MSG_FATAL, MSG_ERROR, MSG_HIGH, MSG_MED, MSG_LOW

DESCRIPTION
  Output a message to be sent to be picked up by the Diag Task.  The
  message levels are controlled by selecting the proper macro function.

    MSG_FATAL   fatal
    MSG_ERROR   error
    MSG_HIGH    high
    MSG_MED     medium
    MSG_LOW     low

  Generation of message generating code is controlled by the setting
  of the MSG_LEVEL #define (defined above and on the 'cl' command line).

PARAMETERS
  x_fmt Format string for message (printf style).  Note, this parameter
        must be a string literal (e.g. "Tuned to %lx"), not a variable.
        This is because the value is compiled into ROM, and can clearly
        not be a dynamic data type.
  a     1st parameter for format string
  b     2nd parameter for format string
  c     3rd parameter for format string

DEPENDENCIES
  msg_init() must be called prior to this macro referencing msg_put().
  This macro, is multiple C expressions, and cannot be used as if it
  is a single expression.

RETURN VALUE
  None

SIDE EFFECTS
  On the target hardware, the file and format strings are placed in ROM.

===========================================================================*/
#if defined (FEATURE_ERR_EXTENDED_STORE) && defined (FEATURE_SAVE_DEBUG_TRACE)

#define MSG_FATAL(x_fmt, a, b, c) \
    MSG_3_SAVE (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_FATAL, x_fmt, a, b, c, TRUE)

#define MSG_ERROR(x_fmt, a, b, c) \
    MSG_3_SAVE (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_ERROR, x_fmt, a, b, c, TRUE)

#define MSG_HIGH(x_fmt, a, b, c) \
    MSG_3_SAVE (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_HIGH, x_fmt, a, b, c, TRUE)

#define MSG_MED(x_fmt, a, b, c) \
    MSG_3_SAVE (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_MED, x_fmt, a, b, c, TRUE)

#define MSG_LOW(x_fmt, a, b, c) \
    MSG_3_SAVE (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_LOW, x_fmt, a, b, c, TRUE)

#else /* if not FEATURE_SAVE_DEBUG_TRACE */

#define MSG_FATAL(x_fmt, a, b, c) \
    MSG_3 (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_FATAL, x_fmt, a, b, c)

#define MSG_ERROR(x_fmt, a, b, c) \
    MSG_3 (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_ERROR, x_fmt, a, b, c)

#define MSG_HIGH(x_fmt, a, b, c) \
    MSG_3 (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_HIGH, x_fmt, a, b, c)

#define MSG_MED(x_fmt, a, b, c) \
    MSG_3 (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_MED, x_fmt, a, b, c)

#define MSG_LOW(x_fmt, a, b, c) \
    MSG_3 (MSG_SSID_ANDROID_KERNEL, MSG_LEGACY_LOW, x_fmt, a, b, c)

#endif /* FEATURE_SAVE_DEBUG_TRACE */


/* -------------------------------------------------------------------------
   Function Definitions
   ------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C"
{
#endif

/*===========================================================================

FUNCTION MSG_SEND_3

DESCRIPTION
  This will build a new style diagnostic Message with 3 parameters. 
  Do not call directly; use macro MSG_* defined in msg.h 
  
  Send a message through diag output services.

DEPENDENCIES
  msg_init() must have been called previously.  A free buffer must
  be available or the message will be ignored (never buffered).

===========================================================================*/
  void msg_send_3 (const msg_const_type * xx_msg_const_ptr, unsigned long xx_arg1,
    unsigned long xx_arg2, unsigned long xx_arg3);

int oemmsg_sd_write(const void *buf, int size);
int oemmsg_process_pkt(int cmd, int sub_id, int sub_cmd);

#ifdef __cplusplus
}
#endif
#endif              /* OEM_MSG_H */

